/*
	Sudoku generator

	(C) Copyright 2005 by Mogens Kjaer, mk@lemo.dk

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>

extern int nfilled,newnfilled,bestndigits;
extern int symmetrical;

extern int displayboard[9][9];

#define RNDDIGIT (random()%9+1)
#define NSET 4

extern int setboard[9][9];
extern int setboardsave[9][9];
extern int beforeboard[9][9];
extern int board[9][9];
extern int nmove,nsol,ntest,running,solved;

static int randomize=1;

static void permute(int n, int *values)
{
  int i,j,tmp;

  for(i=0;i<(n-1);i++)
  {
    j=random()%(n-i)+i;
    tmp=values[i];
    values[i]=values[j];
    values[j]=tmp;
  }
}

static int check(int r, int c, int n)
{
  int i,j;
  for(i=0;i<9;i++)
  { if(setboard[i][c]==n) return 0; if(setboard[r][i]==n) return 0; }
  for(i=0;i<3;i++) for(j=0;j<3;j++)
  { if(setboard[r/3*3+i][c/3*3+j]==n) return 0; }
  return 1;
}

static void move()
{
  int i,j,i1,j1,p,best,bestp,n,r,c,empty;
  int indices[10];
  int nindices;
  nmove++;
  r = -1;
  c = -1;
  best = 999;
  empty = 0;

  for(i=0; i<9; i++)
  {
    for(j=0; j<9; j++)
    {
      if(board[i][j])
      {
	p=0;
	empty++;
      }
      else
      {
	p=1022;
	for(i1=0; i1<9; i1++)
	{
	  p &= ~(1<<board[i][i1]);
	  p &= ~(1<<board[i1][j]);
	}
	for(i1=0;i1<3;i1++)
	for(j1=0;j1<3;j1++)
	  p &= ~(1<<board[i/3*3+i1][j/3*3+j1]);
	/* calculate n as the number of bits set
	   to 1 in p: */
	n = p;
	n = (0x5555 & n) + (0x5555 & (n>> 1));
        n = (0x3333 & n) + (0x3333 & (n>> 2));
	n = (0x0f0f & n) + (0x0f0f & (n>> 4));
	n = (0x00ff & n) + (0x00ff & (n>> 8));
	if(n<best)
	{
	  best=n;
	  bestp=p;
	  r = i;
	  c = j;
	  /* if n is zero, we've reach a dead end */
	  if(n==0) return;
	}
      }
    }
  }

  /* if r is -1, all cells are filled, and we're done */
  if(r== -1)
  {
    nsol++;
    return;
  }
  if(randomize && best>1)
  {
    nindices=0;
    for(i=1;bestp;i++)
    {
      bestp >>= 1;
      if(1 & bestp)
      {
        indices[nindices++]=i;
      }
    }
    permute(nindices, indices);
    for(i=0;i<nindices;i++)
    {
      board[r][c]=indices[i];
      move();
      if(nsol>1) return;
      board[r][c]=0;
    }
  }
  else
  {
    for(i=1;bestp;i++)
    {
      bestp >>= 1;
      if(1 & bestp)
      {
	board[r][c]=i;
	move();
	if(nsol>1) return;
	board[r][c]=0;
      }
    }
  }
}

void sudoku_run()
{
  int i,j,i1,j1,ndigits;
  int i2,j2,k2;
  int walktype;
  char windowtitle[80];

  solved=0;

  ntest++;

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  board[i][j] = 0;

  randomize=1;

  nsol=0;
  move();
  if(nsol==0)
  {
    fprintf(stderr, "Hm, can't happen.\n");
    exit(0);
  }

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  setboardsave[i][j] = board[i][j];

  randomize=0;

  if(symmetrical)
  {
    for(walktype=0;walktype<4;walktype++)
    {
      for(i1=0; i1<9; i1++)
      for(j1=0; j1<9; j1++)
      setboard[i1][j1] = setboardsave[i1][j1];
      ndigits=81;
      for(i2=0;i2<5;i2++)
      for(j2=0;j2<5;j2++)
      {
	i=i2;
	j=j2;
	if(walktype&1) i=4-i;
	if(walktype&2) j=4-j;
	for(i1=0; i1<9; i1++)
	for(j1=0; j1<9; j1++)
	board[i1][j1] = setboard[i1][j1];
	board[4+i][4+j] = 0;
	board[4+i][4-j] = 0;
	board[4-i][4+j] = 0;
	board[4-i][4-j] = 0;
	nsol=0;
	move();
	if(nsol==1)
	{
	  setboard[4+i][4+j] = 0;
	  setboard[4+i][4-j] = 0;
	  setboard[4-i][4+j] = 0;
	  setboard[4-i][4-j] = 0;
	  if((i==0)&&(j==0))
	  {
	    ndigits -= 1;
	  }
	  else if((i==0)||(j==0))
	  {
	    ndigits -= 2;
	  }
	  else
	  {
	    ndigits -= 4;
	  }
	}
      }
      if(ndigits<=bestndigits)
      {
	for(i=0; i<9; i++)
	for(j=0; j<9; j++)
	displayboard[i][j] = setboard[i][j];
	bestndigits=ndigits;
	main_update();
      }
      if(ndigits<=nfilled)
      {
	running=0;
	setcreatemenutext("Create");
	sprintf(windowtitle, "Sudoku Generator. %d cells in %d iteration%s", ndigits, ntest, ntest==1?"":"s");
	settitle(windowtitle);

	main_update();
	return;
      }
    }
  }
  else
  {
    for(walktype=0;walktype<16;walktype++)
    {
      ndigits=81;

      for(i1=0; i1<9; i1++)
      for(j1=0; j1<9; j1++)
      setboard[i1][j1] = setboardsave[i1][j1];

      for(i2=0; i2<9; i2++)
      for(j2=0; j2<9; j2++)
      {
	i=i2;
	j=j2;
	if(walktype&1) i=8-i;
	if(walktype&2) j=8-j;
	if(walktype&4) i=(i+4)%9;
	if(walktype&8) j=(j+4)%9;
	for(i1=0; i1<9; i1++)
	for(j1=0; j1<9; j1++)
	board[i1][j1] = setboard[i1][j1];

	board[i][j] = 0;
	nsol=0;
	move();
	if(nsol==1)
	{
	  setboard[i][j] = 0;
	  ndigits--;
	}
      }
      if(ndigits<=bestndigits)
      {
	for(i=0; i<9; i++)
	for(j=0; j<9; j++)
	displayboard[i][j] = setboard[i][j];
	bestndigits=ndigits;
	main_update();
      }
      if(ndigits<=nfilled)
      {
	running=0;
	setcreatemenutext("Create");
	sprintf(windowtitle, "Sudoku Generator. %d cells in %d iteration%s", ndigits, ntest, ntest==1?"":"s");
	settitle(windowtitle);

	main_update();
	return;
      }
    }
  }
  sprintf(windowtitle, "Sudoku Generator. Iteration %d, filled: %d, best: %d", ntest, ndigits, bestndigits);
  settitle(windowtitle);
}

void unsolve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  displayboard[i][j] = beforeboard[i][j];
  main_update();
  solved=0;
  setsolvemenutext("Solve");
}

void solve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  {
    board[i][j] = displayboard[i][j];
    beforeboard[i][j] = displayboard[i][j];
  }
  randomize=0;
  nsol=1;
  move();
  if(nsol>0)
  {
    for(i=0; i<9; i++)
    for(j=0; j<9; j++)
    displayboard[i][j] = beforeboard[i][j]?board[i][j]:-board[i][j];
    main_update();
    solved=1;
    setsolvemenutext("Unsolve");
  }
}

